/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-21 17:17:03.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.utils;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

/**
 * @author: fengdesheng
 * @since: 2018/11/21
 */
public class ExcelUtils {
    private static Logger log = LoggerFactory.getLogger(ExcelUtils.class);

    public static <T> void exportExcel(Class<T> clazz, List<T> records, File excelFile) {
        exportExcel(clazz, records, excelFile, null);
    }

    public static <T> void exportExcel(Class<T> clazz, List<T> records, File excelFile, String title) {
        if (excelFile == null) {
            log.warn("[ExportExcel] excelFile is null.");
            return;
        }

        if (records == null) {
            log.warn("[ExportExcel] records is null.");
            return;
        }

        log.debug("[ExportExcel] records size is " + records.size());

        ExportParams params = new ExportParams();
        params.setStyle(ExcelStyle.class);
        params.setType(ExcelType.XSSF);

        if (StringUtils.isNotBlank(title)) {
            params.setTitle(title);
            params.setSheetName(title);
        }

        try (FileOutputStream fos = new FileOutputStream(excelFile);
             Workbook workbook = ExcelExportUtil.exportExcel(params, clazz, records);) {

            workbook.write(fos);
        } catch (FileNotFoundException e) {
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

}