/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-01 16:01:30.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.api.controller;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.print.common.Result;
import cn.com.qmth.examcloud.core.print.common.upyun.UpYunClient;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.common.utils.FileUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.File;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * 常用相关接口
 *
 * @author: fengdesheng
 * @since: 2018/11/01
 */
@RestController
@Api(tags = "常用相关接口")
@RequestMapping("${$rmp.ctrl.print}/common")
public class CommonController extends ControllerSupport {
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);
    @Autowired
    private UpYunClient upYunClient;

    @PostMapping("/upload")
    @ApiOperation(value = "上传文件")
    public Result upload(HttpServletRequest request) throws Exception {
        MultipartFile file = null;
        try {
            //暂时只支持单文件上传，默认处理一个文件
            file = ((MultipartHttpServletRequest) request).getFile("file");
        } catch (Exception e) {
            log.warn(e.getMessage());
        }

        Check.isNull(file, "上传的文件不能为空！");
        Check.isTrue(file.isEmpty(), "上传的文件不能为空！");

        String fileUrl = upYunClient.upload(file.getBytes(), file.getOriginalFilename());
        if (StringUtils.isNotEmpty(fileUrl)) {
            return Result.success(fileUrl);
        }

        throw new StatusException(PRT_CODE_500, "上传失败！");
    }

    @GetMapping("/download")
    @ApiOperation(value = "下载文件")
    public void download(@RequestParam String filePath) throws Exception {
        File file = upYunClient.download(filePath);
        if (file != null) {
            super.exportFile(FileUtils.getFileName(filePath), file);
        } else {
            //文件不存在
            super.exportFile("404", new byte[0]);
        }
    }

}