/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-23 17:52:03.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.print.common.jpa.SqlWrapper;
import cn.com.qmth.examcloud.core.print.common.utils.JsonMapper;
import cn.com.qmth.examcloud.core.print.enums.ExamType;
import cn.com.qmth.examcloud.core.print.service.StatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.common.CourseInfo;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamCourseInfo;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamInfo;
import cn.com.qmth.examcloud.core.print.service.bean.examquestionstructure.ExamQuestionStructureInfo;
import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.core.questions.api.request.GetBasePaperReq;
import cn.com.qmth.examcloud.core.questions.api.response.GetQuestionListResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamPaperTypeRelation;
import cn.com.qmth.examcloud.examwork.api.request.CountExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamCoursePaperTypeListReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamOrgListReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamStudentPropertyValueListReq;
import cn.com.qmth.examcloud.examwork.api.response.CountExamStudentResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamCoursePaperTypeListResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamOrgListResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamStudentPropertyValueListResp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author: fengdesheng
 * @since: 2018/11/7
 */
@Service
public class StatisticServiceImpl implements StatisticService {
    private static final Logger log = LoggerFactory.getLogger(StatisticServiceImpl.class);
    @Autowired
    private CourseCloudService courseCloudService;
    @Autowired
    private ExamCloudService examCloudService;
    @Autowired
    private ExtractConfigCloudService questionCloudService;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    public List<ExamCourseInfo> findExamCourseAndPaperTypes(Long orgId, Long examId) {
        List<ExamCourseInfo> coursePaperTypes = new ArrayList<>();
        GetExamCoursePaperTypeListReq req = new GetExamCoursePaperTypeListReq();
        req.setRootOrgId(orgId);
        req.setExamId(examId);
        Long start = 0L;
        while (true) {
            req.setStart(start);
            GetExamCoursePaperTypeListResp resp = examCloudService.getExamCoursePaperTypeList(req);
            List<ExamPaperTypeRelation> list = resp.getRelationList();
            if (list == null || list.isEmpty()) {
                break;
            }

            for (ExamPaperTypeRelation relation : list) {
                coursePaperTypes.add(new ExamCourseInfo(orgId, relation.getExamId(), relation.getCourseId(), relation.getPaperType()));
            }

            if (start.equals(resp.getNext())) {
                break;
            } else {
                start = resp.getNext();
            }
        }
        return coursePaperTypes;
    }

    @Override
    public int countExamTotalCourse(List<ExamCourseInfo> coursePaperTypes) {
        if (coursePaperTypes == null || coursePaperTypes.isEmpty()) {
            return 0;
        }
        //获取考试的实际课程数量
        Set<Long> ids = new HashSet<>();
        for (ExamCourseInfo info : coursePaperTypes) {
            ids.add(info.getCourseId());
        }
        return ids.size();
    }

    @Override
    public List<String> findExamPackageCodes(Long orgId, Long examId) {
        GetExamStudentPropertyValueListReq req = new GetExamStudentPropertyValueListReq();
        req.setExamId(examId);
        req.setPropertyName("ext1");//试卷袋编号字段
        return this.findExamStudentProperties(req);
    }

    @Override
    public List<String> findExamSites(Long examId) {
        GetExamStudentPropertyValueListReq req = new GetExamStudentPropertyValueListReq();
        req.setExamId(examId);
        req.setPropertyName("examSite");//考点字段
        return this.findExamStudentProperties(req);
    }

    @Override
    public List<Long> findExamLearnCenters(Long examId) {
        List<Long> ids = new ArrayList<>();
        GetExamOrgListReq req = new GetExamOrgListReq();
        req.setExamId(examId);
        Long start = 0L;
        while (true) {
            req.setStart(start);
            GetExamOrgListResp resp = examCloudService.getExamOrgList(req);
            List<Long> list = resp.getOrgIdList();
            if (list == null || list.isEmpty()) {
                break;
            }
            ids.addAll(list);
            if (start.equals(resp.getNext())) {
                break;
            } else {
                start = resp.getNext();
            }
        }
        return ids;
    }

    @Override
    public int countExamTotalStudentByExamId(Long examId) {
        CountExamStudentReq req = new CountExamStudentReq();
        req.setExamId(examId);
        return this.countExamTotalStudent(req);
    }

    @Override
    public int countExamTotalStudentByCourseIdAndPaperType(Long examId, Long courseId, String paperType) {
        CountExamStudentReq req = new CountExamStudentReq();
        req.setExamId(examId);
        req.setCourseId(courseId);
        req.setPaperType(paperType);
        return this.countExamTotalStudent(req);
    }

    @Override
    public int countExamTotalStudentByPackageCode(Long examId, String packageCode) {
        CountExamStudentReq req = new CountExamStudentReq();
        req.setExamId(examId);
        req.setExt1(packageCode);//试卷袋编号
        return this.countExamTotalStudent(req);
    }

    @Override
    public int countExamTotalStudentByLearnCenter(Long examId, Long learnCenterId) {
        CountExamStudentReq req = new CountExamStudentReq();
        req.setExamId(examId);
        req.setOrgId(learnCenterId);//学习中心
        return this.countExamTotalStudent(req);
    }

    @Override
    public int countExamTotalStudentBySite(Long examId, String site) {
        CountExamStudentReq req = new CountExamStudentReq();
        req.setExamId(examId);
        req.setExamSite(site);//考点
        return this.countExamTotalStudent(req);
    }

    private int countExamTotalStudent(CountExamStudentReq req) {
        CountExamStudentResp resp = examCloudService.countExamStudent(req);
        return resp.getCount().intValue();
    }

    private List<String> findExamStudentProperties(GetExamStudentPropertyValueListReq req) {
        List<String> properties = new ArrayList<>();
        String start = "";
        while (true) {
            req.setStart(start);
            GetExamStudentPropertyValueListResp resp = examCloudService.getExamStudentPropertyValueList(req);
            List<String> list = resp.getValueList();
            if (list == null || list.isEmpty()) {
                break;
            }
            properties.addAll(list);
            String last = list.get(list.size() - 1);
            if (start.equals(last)) {
                break;
            } else {
                start = last;
            }
        }
        return properties;
    }

    @Override
    public CourseInfo findCourseInfo(Long orgId, Long courseId) {
        GetCourseReq req = new GetCourseReq();
        req.setRootOrgId(orgId);
        req.setId(courseId);
        GetCourseResp resp = courseCloudService.getCourse(req);
        CourseBean bean = resp.getCourseBean();
        return new CourseInfo(bean.getId(), bean.getCode(), bean.getName());
    }

    @Override
    public ExamQuestionStructureInfo findStructureByPaperId(Long examId, String paperId) {
        GetBasePaperReq req = new GetBasePaperReq();
        req.setPaperId(paperId);
        GetQuestionListResp resp = questionCloudService.getQuestionStructure(req);
        return new JsonMapper().fromJson(resp.getJson(), ExamQuestionStructureInfo.class);
    }

    @Override
    public List<ExamInfo> findTraditionExams() {
        SqlWrapper sql = new SqlWrapper()
                .select("em.id examId,em.name examName,em.root_org_id orgId,org.name orgName").from("ec_e_exam em")
                .innerJoin("ec_b_org org").on("org.id", "em.root_org_id")
                .where().eq("em.exam_type", ExamType.TRADITION.name())
                .orderBy("em.id", false);
        return jdbcTemplate.query(sql.build(), new BeanPropertyRowMapper(ExamInfo.class));
    }

}